/*
 * KPlayer.cpp
 *
 *  Created on: Oct 2, 2008
 *      Author: Santi Ontan
 */

#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include <windows.h>
#endif

#include "debug.h"
#include "Knightmare.h"
#include "KObjectCreator.h"
#include "KPlayer.h"
#include "Weapon.h"
#include "WeaponArrow.h"
#include "WeaponDArrow.h"
#include "WeaponKnife.h"
#include "WeaponDKnife.h"
#include "WeaponBoomerang.h"
#include "KText.h"

#include "VirtualController.h"

KPlayer::KPlayer(int x,int y,int sfx_volume,int ID) : KObject(x,y,sfx_volume)
{
	int i;

	m_ID = ID;

	m_class = kplayer_symbol;
	m_move_speed = 1.00f;
	m_weapon = WEAPON_ARROW;
	m_weapon_count = 0;

	m_keep_angle = true;

	m_shadow_tile = 0;
	m_shadow_offset = 26;

    m_n_states = 8;
    m_state_animations = new Animation *[8];

    for(i=0;i<8;i++) m_state_animations[i]=0;	
}

KPlayer::~KPlayer() {
}


bool KPlayer::is_a(Symbol *c)
{
    if (c->cmp(kplayer_symbol))
        return true;

    return KObject::is_a(c);
} /* KPlayer::is_a */


bool KPlayer::is_a(char *c)
{
    bool retval;
    Symbol *s = new Symbol(c);

    retval = is_a(s);

    delete s;

    return retval;
} /* KPlayer::is_a */


bool KPlayer::cycle(List<VirtualController> *vcl, Knightmare *game, GLTManager *GLTM, SFXManager *SFXM)
{
	float desired_x_move = 0;
	float desired_y_move = 0;
	
	if (!KObject::cycle(vcl,game,GLTM,SFXM)) return false;
			
//	output_debug_message("Player: %g\n",m_y);

	VirtualController *k = 0, *ktmp;
	vcl->Rewind();
	while(vcl->Iterate(ktmp)) {
		if (ktmp->m_ID == m_ID) k = ktmp;
	}
	
	// CHEAT CODE:
	if (k->m_keyboardstate->key_pressed(SDLK_k) &&
		k->m_keyboardstate->key_pressed(SDLK_n) &&
		k->m_keyboardstate->key_press(SDLK_i)) {
		// change weapon:
		m_weapon++;
		if (m_weapon>=N_WEAPONS) m_weapon=0;
		weaponChangeEffect(m_weapon,game,SFXM);
	}	

/*
	output_debug_message("k: %s %s %s %s\n",(k->m_joystick[VC_LEFT] ? "true":"false"),
			(k->m_joystick[VC_RIGHT] ? "true":"false"),
			(k->m_joystick[VC_UP] ? "true":"false"),
			(k->m_joystick[VC_DOWN] ? "true":"false"));
*/

	m_state = PLAYER_STATE_IDLE;
	if (k->m_joystick[VC_LEFT]) {
		desired_x_move-=m_move_speed;
		m_state = PLAYER_STATE_WALKING;
	}
	if (k->m_joystick[VC_RIGHT]) {
		desired_x_move+=m_move_speed;
		m_state = PLAYER_STATE_WALKING;
	}
	if (k->m_joystick[VC_UP]) {
		desired_y_move-=m_move_speed;
		m_state = PLAYER_STATE_WALKING;
	}
	if (k->m_joystick[VC_DOWN]) {
		desired_y_move+=m_move_speed;
		m_state = PLAYER_STATE_WALKING;
	}
	
	// attempt to apply the movement:
	if (desired_x_move!=0 || desired_y_move!=0) {
		if (game->get_map()->collisionWithTypes(this,desired_x_move,desired_y_move,game->getObstacleClasses())) {
			if (game->get_map()->collisionWithTypes(this,desired_x_move,0,game->getObstacleClasses())) {
				if (game->get_map()->collisionWithTypes(this,0,desired_y_move,game->getObstacleClasses())) {
					// no move is possible!
				} else {
					m_y += desired_y_move;			
				}
			} else {
				m_x += desired_x_move;			
			}	
		} else {
			m_x += desired_x_move;
			m_y += desired_y_move;
		}
	}
	
	// Free the character in case it's trapped:
	if (game->get_map()->collisionWithTypes(this,desired_x_move,desired_y_move,game->getObstacleClasses())) {
		int max_move = 4;
		int i,j,xo,yo;
		int xm[8]={0,0,1,-1, 1,-1,1,-1};
		int ym[8]={-1,1,0,0, 1,1,-1,-1};
		bool found = false;
		
		for(i = 1;!found && i<max_move;i++) {
			for(j=0;!found && j<8;j++) {
				xo = i*xm[j];
				yo = j*ym[j];
				if (!game->get_map()->collisionWithTypes(this,xo,yo,game->getObstacleClasses())) {
					m_x+=xo;
					m_y+=yo;
					found = true;
				}
			}
		}
	}
	
	// Weapons:
	if (k->m_button[0] && !k->m_old_button[0]) {
		switch(m_weapon) {
			case WEAPON_ARROW:
				if (m_weapon_count<2) {
					Weapon *w = new WeaponArrow(m_x,m_y,m_sfx_volume,GLTM,this);
					SFXM->SFX_play("sfx/player_arrow1", m_sfx_volume);
					game->get_map()->addObject(3,w);
					m_weapon_count++;
				}
				break;
			case WEAPON_D_ARROW:
				if (m_weapon_count<2) {
					Weapon *w = new WeaponDArrow(m_x,m_y,m_sfx_volume,GLTM,this);
					game->get_map()->addObject(3,w);
					m_weapon_count++;
				}
				break;
			case WEAPON_KNIFE:
				if (m_weapon_count<3) {
					Weapon *w = new WeaponKnife(m_x,m_y,m_sfx_volume,GLTM,this);
					game->get_map()->addObject(3,w);
					m_weapon_count++;
				}
				break;
			case WEAPON_D_KNIFE:
				if (m_weapon_count<3) {
					Weapon *w = new WeaponDKnife(m_x,m_y,m_sfx_volume,GLTM,this);
					game->get_map()->addObject(3,w);
					m_weapon_count++;
				}
				break;
			case WEAPON_FIREBALL1:
				if (m_weapon_count==0) {
				}
				break;
			case WEAPON_FIREBALL2:
				if (m_weapon_count==0) {
				}
				break;
			case WEAPON_BOOMERANG1:
				if (m_weapon_count<2) {
					Weapon *w = new WeaponBoomerang(m_x,m_y-8,m_sfx_volume,GLTM,this);
					game->get_map()->addObject(3,w);
					m_weapon_count++;
				}
				break;
			case WEAPON_BOOMERANG2:
				if (m_weapon_count<3) {
					Weapon *w = new WeaponBoomerang(m_x,m_y-8,m_sfx_volume,GLTM,this);
					game->get_map()->addObject(3,w);
					m_weapon_count++;
				}
				break;
		}
	}
		
	return true;
}


void KPlayer::weaponDisappeared(Weapon *w)
{
	if (m_weapon_count>0) m_weapon_count--;
}


void KPlayer::weaponChangeEffect(int weapon,Knightmare *game,SFXManager *SFXM)
{
	char *weapon_names[N_WEAPONS]={"Arrows","Double Arrows",
								   "Knives","Double Knives",
								   "Spread Fireballs","Forward Fireballs",
								   "Boomerang","Triple Boomerang"};
	game->get_map()->addObject(7, new KText(m_x+20,m_y-20,weapon_names[m_weapon],game->getInGameFont(),50));
}


void KPlayer::draw(float angle,int x_offset,int y_offset,GLTManager *GLTM)
{
	// draw shadow:
	if (m_shadow_tile!=0) 
		m_shadow_tile->draw(m_x+x_offset,m_y+y_offset+m_shadow_offset,0,0,1);

	KObject::draw(angle,x_offset,y_offset,GLTM);
} /* KPlayer::draw */

